@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2013-2019 Laboratoire de Recherche et Développement
+// Copyright (C) 2013-2020 Laboratoire de Recherche et Développement
 // de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -133,6 +133,7 @@ enum {
   OPT_MASK_ACC,
   OPT_MERGE,
   OPT_NONDET_STATES,
+  OPT_PARTIAL_DEGEN,
   OPT_PRODUCT_AND,
   OPT_PRODUCT_OR,
   OPT_RANDOMIZE,
@@ -266,6 +267,9 @@ static const argp_option options[] =
       "keep automata that accept WORD", 0 },
     { "reject-word", OPT_REJECT_WORD, "WORD", 0,
       "keep automata that reject WORD", 0 },
+    { "nth", 'N', "RANGE", 0,
+      "assuming input automata are numbered from 1, keep only those in RANGE",
+      0 },
     /**************************************************/
     RANGE_DOC_FULL,
     WORD_DOC,
@@ -363,6 +367,10 @@ static const argp_option options[] =
     { "sum-and", OPT_SUM_AND, "FILENAME", 0,
       "build the sum with the automaton in FILENAME "
       "to intersect languages", 0 },
+    { "partial-degeneralize", OPT_PARTIAL_DEGEN, "NUM1,NUM2,...",
+      OPTION_ARG_OPTIONAL, "Degeneralize automata according to sets "
+      "NUM1,NUM2,... If no sets are given, partial degeneralization "
+      "is performed for all conjunctions of Inf and disjunctions of Fin.", 0 },
     { "separate-sets", OPT_SEP_SETS, nullptr, 0,
       "if both Inf(x) and Fin(x) appear in the acceptance condition, replace "
       "Fin(x) by a new Fin(y) and adjust the automaton", 0 },
@@ -419,8 +427,31 @@ static const struct argp_child children[] =
   };
 
 
-typedef spot::twa_graph::graph_t::edge_storage_t tr_t;
-typedef std::set<std::vector<tr_t>> unique_aut_t;
+struct canon_aut
+{
+  typedef spot::twa_graph::graph_t::edge_storage_t tr_t;
+  unsigned num_states;
+  std::vector<tr_t> edges;
+  std::string acc;
+
+  canon_aut(const spot::const_twa_graph_ptr& aut)
+    : num_states(aut->num_states())
+    , edges(aut->edge_vector().begin() + 1,
+            aut->edge_vector().end())
+  {
+    std::ostringstream os;
+    aut->get_acceptance().to_text(os);
+    acc = os.str();
+  }
+
+  bool operator<(const canon_aut& o) const
+  {
+    return std::tie(num_states, edges, acc)
+      < std::tie(o.num_states, o.edges, o.acc);
+  };
+};
+
+typedef std::set<canon_aut> unique_aut_t;
 static long int match_count = 0;
 static spot::option_map extra_options;
 static bool randomize_st = false;
@@ -607,6 +638,7 @@ static bool opt_terminal_sccs_set = false;
 static range opt_nondet_states = { 0, std::numeric_limits<int>::max() };
 static bool opt_nondet_states_set = false;
 static int opt_max_count = -1;
+static range opt_nth = { 0, std::numeric_limits<int>::max() };
 static bool opt_destut = false;
 static char opt_instut = 0;
 static bool opt_is_empty = false;
@@ -619,6 +651,8 @@ static bool opt_complement = false;
 static bool opt_complement_acc = false;
 static char* opt_decompose_scc = nullptr;
 static bool opt_dualize = false;
+static bool opt_partial_degen_set = false;
+static spot::acc_cond::mark_t opt_partial_degen = {};
 static spot::acc_cond::mark_t opt_mask_acc = {};
 static std::vector<bool> opt_keep_states = {};
 static unsigned int opt_keep_states_initial = 0;
@@ -702,9 +736,11 @@ parse_opt(int key, char* arg, struct argp_state*)
     case 'n':
       opt_max_count = to_pos_int(arg, "-n/--max-count");
       break;
+    case 'N':
+      opt_nth = parse_range(arg, 0, std::numeric_limits<int>::max());
+      break;
     case 'u':
-      opt->uniq =
-        std::unique_ptr<unique_aut_t>(new std::set<std::vector<tr_t>>());
+      opt->uniq = std::unique_ptr<unique_aut_t>(new std::set<canon_aut>());
       break;
     case 'v':
       opt_invert = true;
@@ -1005,6 +1041,23 @@ parse_opt(int key, char* arg, struct argp_state*)
       opt_nondet_states = parse_range(arg, 0, std::numeric_limits<int>::max());
       opt_nondet_states_set = true;
       break;
+    case OPT_PARTIAL_DEGEN:
+      {
+        opt_partial_degen_set = true;
+        if (arg)
+          for (auto res : to_longs(arg))
+            {
+              if (res < 0)
+                error(2, 0, "acceptance sets should be non-negative:"
+                      " --partial-degeneralize=%ld", res);
+              if (static_cast<unsigned long>(res) >=
+                  spot::acc_cond::mark_t::max_accsets())
+                error(2, 0, "this implementation does not support that many"
+                      " acceptance sets: --partial-degeneralize=%ld", res);
+              opt_partial_degen.set(res);
+            }
+        break;
+      }
     case OPT_PRODUCT_AND:
       {
         auto a = read_automaton(arg, opt->dict);
@@ -1286,6 +1339,9 @@ namespace
     int
     process_automaton(const spot::const_parsed_aut_ptr& haut) override
     {
+      static unsigned order = 0;
+      ++order;
+
       spot::process_timer timer;
       timer.start();
 
@@ -1326,6 +1382,7 @@ namespace
 
       bool matched = true;
 
+      matched &= opt_nth.contains(order);
       matched &= opt_states.contains(aut->num_states());
       matched &= opt_edges.contains(aut->num_edges());
       matched &= opt_accsets.contains(aut->acc().num_sets());
@@ -1478,6 +1535,19 @@ namespace
       else if (opt_rem_unreach)
         aut->purge_unreachable_states();
 
+      if (opt_partial_degen_set)
+        {
+          if (opt_partial_degen)
+            {
+              auto sets = opt_partial_degen & aut->acc().all_sets();
+              aut = spot::partial_degeneralize(aut, sets);
+            }
+          else
+            {
+              aut = spot::partial_degeneralize(aut);
+            }
+        }
+
       if (opt->product_and)
         aut = ::product(std::move(aut), opt->product_and);
       if (opt->product_or)
@@ -1559,8 +1629,7 @@ namespace
           auto tmp =
             spot::canonicalize(make_twa_graph(aut,
                                                  spot::twa::prop_set::all()));
-          if (!opt->uniq->emplace(tmp->edge_vector().begin() + 1,
-                                  tmp->edge_vector().end()).second)
+          if (!opt->uniq->emplace(tmp).second)
             return 0;
         }
 
