@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2012 Laboratoire de Recherche et Développement de
+// Copyright (C) 2012, 2016 Laboratoire de Recherche et Développement de
 // l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -20,18 +20,42 @@
 #include "common_sys.hh"
 #include "common_cout.hh"
 #include <iostream>
+#include <chrono>
 #include "error.h"
 
+namespace
+{
+  static std::chrono::steady_clock::time_point last_flush;
+
+  static void do_check_cout()
+  {
+    // Make sure we abort if we can't write to std::cout anymore
+    // (like disk full or broken pipe with SIGPIPE ignored).
+    if (!std::cout)
+      error(2, 0, "error writing to standard output");
+  }
+}
+
 void check_cout()
 {
-  // Make sure we abort if we can't write to std::cout anymore
-  // (like disk full or broken pipe with SIGPIPE ignored).
-  if (!std::cout)
-    error(2, 0, "error writing to standard output");
+  // If we haven't flushed explicitly for more than 20ms, do it now.
+  // Otherwise we would have to wait for the buffer to fill up, and
+  // this could take a long time.
+  auto now = std::chrono::steady_clock::now();
+  auto ms =
+    std::chrono::duration_cast<std::chrono::milliseconds>(now - last_flush);
+  if (ms.count() >= 20)
+    {
+      last_flush = now;
+      std::cout.flush();
+    }
+
+  do_check_cout();
 }
 
 void flush_cout()
 {
+  last_flush = std::chrono::steady_clock::now();
   std::cout.flush();
-  check_cout();
+  do_check_cout();
 }
