@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2012-2019 Laboratoire de Recherche et Développement
+// Copyright (C) 2012-2020 Laboratoire de Recherche et Développement
 // de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -98,6 +98,7 @@ enum {
   OPT_OMIT,
   OPT_PRODUCTS,
   OPT_REFERENCE,
+  OPT_SAVE_INCLUSION_PRODUCTS,
   OPT_SEED,
   OPT_STATES,
   OPT_STOP_ERR,
@@ -180,6 +181,9 @@ static const argp_option options[] =
       "suppress all normal output in absence of error", 0 },
     { "save-bogus", OPT_BOGUS, "[>>]FILENAME", 0,
       "save formulas for which problems were detected in FILENAME", 0 },
+    { "save-inclusion-products",  OPT_SAVE_INCLUSION_PRODUCTS, "[>>]FILENAME",
+      0, "save automata representing products built to check inclusion "
+      "between automata", 0 },
     { "verbose", OPT_VERBOSE, nullptr, 0,
       "print what is being done, for debugging", 0 },
     { nullptr, 0, nullptr, 0,
@@ -220,7 +224,10 @@ static bool products_avg = true;
 static bool opt_omit = false;
 static const char* bogus_output_filename = nullptr;
 static output_file* bogus_output = nullptr;
+static const char* grind_output_filename = nullptr;
 static output_file* grind_output = nullptr;
+static const char* saved_inclusion_products_filename = nullptr;
+static output_file* saved_inclusion_products = nullptr;
 static bool verbose = false;
 static bool quiet = false;
 static bool ignore_exec_fail = false;
@@ -507,6 +514,7 @@ parse_opt(int key, char* arg, struct argp_state*)
       fail_on_timeout = true;
       break;
     case OPT_GRIND:
+      grind_output_filename = arg;
       grind_output = new output_file(arg);
       break;
     case OPT_IGNORE_EXEC_FAIL:
@@ -539,6 +547,12 @@ parse_opt(int key, char* arg, struct argp_state*)
     case OPT_REFERENCE:
       tools_push_trans(arg, true);
       break;
+    case OPT_SAVE_INCLUSION_PRODUCTS:
+      {
+        saved_inclusion_products = new output_file(arg);
+        saved_inclusion_products_filename = arg;
+        break;
+      }
     case OPT_SEED:
       seed = to_pos_int(arg, "--seed");
       break;
@@ -582,10 +596,14 @@ namespace
       format(command, tools[translator_num].cmd);
 
       std::string cmd = command.str();
-      auto disp_cmd = [&]() {
-                        std::cerr << "Running [" << l << translator_num
-                                  << "]: " << cmd << '\n';
-                      };
+      auto disp_cmd =
+        [&]() {
+          std::cerr << "Running [" << l << translator_num;
+          const char* name = tools[translator_num].name;
+          if (name != tools[translator_num].spec)
+            std::cerr << ": " << name;
+          std::cerr << "]: " << cmd << '\n';
+        };
       if (!quiet)
         disp_cmd();
       spot::process_timer timer;
@@ -754,7 +772,8 @@ namespace
   static bool
   check_empty_prod(const spot::const_twa_graph_ptr& aut_i,
                    const spot::const_twa_graph_ptr& aut_j,
-                   size_t i, size_t j, bool icomp, bool jcomp)
+                   size_t i, size_t j, bool icomp, bool jcomp,
+                   const std::string& formula)
   {
     if (aut_i->num_sets() + aut_j->num_sets() >
         spot::acc_cond::mark_t::max_accsets())
@@ -781,6 +800,22 @@ namespace
 
     auto prod = spot::product(aut_i, aut_j);
 
+    if (saved_inclusion_products)
+      {
+        std::ostringstream os;
+        if (icomp)
+          os << "Comp(N" << i << ')';
+        else
+          os << 'P' << i;
+        if (jcomp)
+          os << "*Comp(P" << j << ')';
+        else
+          os << "*N" << j;
+        os << ", " << formula;
+        prod->set_named_prop("automaton-name", new std::string(os.str()));
+        spot::print_hoa(saved_inclusion_products->ostream(), prod) << std::endl;
+      }
+
     if (verbose)
       {
         std::cerr << "info: check_empty ";
@@ -1243,6 +1278,33 @@ namespace
               unalt(neg, i, 'N');
             }
 
+          print_first = verbose;
+          auto tmp = [&](std::vector<spot::twa_graph_ptr>& x, unsigned i,
+                         const char* prefix, const char* suffix)
+            {
+              if (!x[i])
+                return;
+              unsigned before = x[i]->num_sets();
+              cleanup_acceptance_here(x[i]);
+              unsigned after = x[i]->num_sets();
+              if (verbose && before != after)
+                {
+                  if (print_first)
+                    {
+                      std::cerr << "info: removing unused sets...\n";
+                      print_first = false;
+                    }
+                  std::cerr << "info:   " << prefix << i
+                            << suffix << '\t' << before << " sets -> "
+                            << after << " sets\n";
+                }
+            };
+          for (size_t i = 0; i < m; ++i)
+            {
+              tmp(pos, i, "     P", " ");
+              tmp(neg, i, "     N", " ");
+            }
+
           // Complement
           if (!no_complement)
             {
@@ -1303,67 +1365,11 @@ namespace
                 }
             }
 
-          print_first = verbose;
-          auto tmp = [&](std::vector<spot::twa_graph_ptr>& x, unsigned i,
-                         const char* prefix, const char* suffix)
-            {
-              if (!x[i])
-                return;
-              if (x[i]->acc().uses_fin_acceptance())
-                {
-                  if (verbose)
-                    {
-                      if (print_first)
-                        {
-                          std::cerr <<
-                            "info: getting rid of any Fin acceptance...\n";
-                          print_first = false;
-                        }
-                      std::cerr << "info:\t" << prefix << i
-                                << suffix << "\t(";
-                      printsize(x[i]);
-                      std::cerr << ") ->";
-                    }
-                  try
-                    {
-                      x[i] = remove_fin(x[i]);
-                    }
-                  catch (const std::runtime_error& err)
-                    {
-                      if (verbose)
-                        std::cerr << " failed (" << err.what() << ")\n";
-                      else
-                        std::cerr << "info: preprocessing "
-                                  << prefix << i << suffix
-                                  << " failed (" << err.what() << ")\n";
-                      x[i] = nullptr;
-                    }
-                  if (verbose && x[i])
-                    {
-                      std::cerr << " (";
-                      printsize(x[i]);
-                      std::cerr << ")\n";
-                    }
-                }
-              else
-                {
-                  // Remove useless sets nonetheless.
-                  cleanup_acceptance_here(x[i]);
-                }
-            };
-          for (size_t i = 0; i < m; ++i)
-            {
-              tmp(pos, i, "     P", " ");
-              tmp(neg, i, "     N", " ");
-              tmp(comp_pos, i, "Comp(P", ")");
-              tmp(comp_neg, i, "Comp(N", ")");
-            }
-
           if (smallest_pos_ref >= 0)
             {
-              // Recompute the smallest references now, because removing
-              // alternation and Fin acceptance might have changed the
-              // sizes.
+              // Recompute the smallest references now, because
+              // removing alternation and useless acceptance sets
+              // might have changed the sizes.
               smallest_pos_ref = smallest_ref(pos);
               smallest_neg_ref = smallest_ref(neg);
 
@@ -1392,7 +1398,8 @@ namespace
                               smallest_pos_ref >= 0 &&
                               (size_t)smallest_pos_ref != i)))
                       problems +=
-                        check_empty_prod(pos[i], neg[j], i, j, false, false);
+                        check_empty_prod(pos[i], neg[j], i, j, false, false,
+                                         fstr);
 
                     // Deal with the extra complemented automata if we
                     // have some.
@@ -1413,17 +1420,17 @@ namespace
                       if (smallest_pos_ref < 0 || i == (size_t)smallest_pos_ref)
                         problems +=
                           check_empty_prod(pos[i], comp_pos[j],
-                                           i, j, false, true);
+                                           i, j, false, true, fstr);
                     if (i != j && comp_neg[i] && !comp_pos[i])
                       if (smallest_neg_ref < 0 || j == (size_t)smallest_neg_ref)
                         problems +=
                           check_empty_prod(comp_neg[i], neg[j],
-                                           i, j, true, false);
+                                           i, j, true, false, fstr);
                     if (comp_pos[i] && comp_neg[j] &&
                         (i == j || (!comp_neg[i] && !comp_pos[j])))
                       problems +=
                         check_empty_prod(comp_neg[j], comp_pos[i],
-                                         j, i, true, true);
+                                         j, i, true, true, fstr);
                   }
         }
       else
@@ -1800,8 +1807,21 @@ main(int argc, char** argv)
             }
         }
 
-      delete bogus_output;
-      delete grind_output;
+      if (bogus_output)
+        {
+          bogus_output->close(bogus_output_filename);
+          delete bogus_output;
+        }
+      if (grind_output)
+        {
+          grind_output->close(grind_output_filename);
+          delete grind_output;
+        }
+      if (saved_inclusion_products)
+        {
+          saved_inclusion_products->close(saved_inclusion_products_filename);
+          delete saved_inclusion_products;
+        }
 
       if (json_output)
         print_stats_json(json_output);
