@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2017-2019 Laboratoire de Recherche et Développement
+// Copyright (C) 2017-2020 Laboratoire de Recherche et Développement
 // de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -33,7 +33,9 @@
 #include "common_sys.hh"
 
 #include <spot/misc/bddlt.hh>
+#include <spot/misc/escape.hh>
 #include <spot/misc/game.hh>
+#include <spot/misc/timer.hh>
 #include <spot/tl/formula.hh>
 #include <spot/twa/twagraph.hh>
 #include <spot/twaalgos/aiger.hh>
@@ -51,6 +53,7 @@
 enum
 {
   OPT_ALGO = 256,
+  OPT_CSV,
   OPT_INPUT,
   OPT_OUTPUT,
   OPT_PRINT,
@@ -71,13 +74,14 @@ static const argp_option options[] =
       " propositions", 0},
     /**************************************************/
     { nullptr, 0, nullptr, 0, "Fine tuning:", 10 },
-    { "algo", OPT_ALGO, "ds|sd|large", 0,
+    { "algo", OPT_ALGO, "ds|sd|lar|lar.old", 0,
       "choose the algorithm for synthesis:\n"
       " - sd:   split then determinize with Safra (default)\n"
       " - ds:   determinize (Safra) then split\n"
       " - lar:  translate to a deterministic automaton with arbitrary"
       " acceptance condition, then use LAR to turn to parity,"
-      " then split", 0 },
+      " then split\n"
+      " - lar.old:  old version of LAR, for benchmarking.\n", 0 },
     /**************************************************/
     { nullptr, 0, nullptr, 0, "Output options:", 20 },
     { "print-pg", OPT_PRINT, nullptr, 0,
@@ -88,6 +92,10 @@ static const argp_option options[] =
       "prints the winning strategy as an AIGER circuit", 0},
     { "verbose", OPT_VERBOSE, nullptr, 0,
       "verbose mode", -1 },
+    { "csv", OPT_CSV, "[>>]FILENAME", OPTION_ARG_OPTIONAL,
+      "output statistics as CSV in FILENAME or on standard output "
+      "(if '>>' is used to request append mode, the header line is "
+      "not output)", 0 },
     /**************************************************/
     { nullptr, 0, nullptr, 0, "Miscellaneous options:", -1 },
     { nullptr, 0, nullptr, 0, nullptr, 0 },
@@ -112,22 +120,40 @@ Exit status:\n\
 static std::vector<std::string> input_aps;
 static std::vector<std::string> output_aps;
 
-bool opt_print_pg(false);
-bool opt_real(false);
-bool opt_print_aiger(false);
+static const char* opt_csv = nullptr;
+static bool opt_print_pg = false;
+static bool opt_real = false;
+static bool opt_print_aiger = false;
+
+static double trans_time = 0.0;
+static double split_time = 0.0;
+static double paritize_time = 0.0;
+static double bgame_time = 0.0;
+static double solve_time = 0.0;
+static double strat2aut_time = 0.0;
 
 enum solver
 {
   DET_SPLIT,
   SPLIT_DET,
   LAR,
+  LAR_OLD,
 };
 
+static char const *const solver_names[] =
+  {
+   "ds",
+   "sd",
+   "lar",
+   "lar.old"
+  };
+
 static char const *const solver_args[] =
 {
   "detsplit", "ds",
   "splitdet", "sd",
   "lar",
+  "lar.old",
   nullptr
 };
 static solver const solver_types[] =
@@ -135,12 +161,14 @@ static solver const solver_types[] =
   DET_SPLIT, DET_SPLIT,
   SPLIT_DET, SPLIT_DET,
   LAR,
+  LAR_OLD,
 };
 ARGMATCH_VERIFY(solver_args, solver_types);
 
 static solver opt_solver = SPLIT_DET;
 static bool verbose = false;
 
+
 namespace
 {
 
@@ -260,6 +288,49 @@ namespace
     return aut;
   }
 
+  static void
+  print_csv(spot::formula f, bool realizable)
+  {
+    if (verbose)
+      std::cerr << "writing CSV to " << opt_csv << '\n';
+
+    output_file outf(opt_csv);
+    std::ostream& out = outf.ostream();
+
+    // Do not output the header line if we append to a file.
+    // (Even if that file was empty initially.)
+    if (!outf.append())
+      {
+        out << ("\"formula\",\"algo\",\"trans_time\","
+                "\"split_time\",\"todpa_time\",\"build_game_time\"");
+        if (!opt_print_pg)
+          {
+            out << ",\"solve_time\"";
+            if (!opt_real)
+              out << ",\"strat2aut_time\"";
+            out << ",\"realizable\"";
+          }
+        out << '\n';
+      }
+    std::ostringstream os;
+    os << f;
+    spot::escape_rfc4180(out << '"', os.str());
+    out << "\",\"" << solver_names[opt_solver]
+        << "\"," << trans_time
+        << ',' << split_time
+        << ',' << paritize_time
+        << ',' << bgame_time;
+    if (!opt_print_pg)
+      {
+        out << ',' << solve_time;
+        if (!opt_real)
+          out << ',' << strat2aut_time;
+        out << ',' << realizable;
+      }
+    out << '\n';
+    outf.close(opt_csv);
+  }
+
   class ltl_processor final : public job_processor
   {
   private:
@@ -276,21 +347,22 @@ namespace
     {
     }
 
-    int process_formula(spot::formula f,
-                        const char*, int) override
+    int solve_formula(spot::formula f)
     {
       spot::process_timer timer;
       timer.start();
+      spot::stopwatch sw;
+      bool want_time = verbose || opt_csv;
 
-      if (opt_solver == LAR)
+      if (opt_solver == LAR || opt_solver == LAR_OLD)
         {
           trans_.set_type(spot::postprocessor::Generic);
           trans_.set_pref(spot::postprocessor::Deterministic);
         }
 
+      if (want_time)
+        sw.start();
       auto aut = trans_.run(&f);
-      if (verbose)
-        std::cerr << "translating formula done\n";
       bdd all_inputs = bddtrue;
       bdd all_outputs = bddtrue;
       for (unsigned i = 0; i < input_aps_.size(); ++i)
@@ -309,34 +381,57 @@ namespace
           unsigned v = aut->register_ap(spot::formula::ap(lowercase.str()));
           all_outputs &= bdd_ithvar(v);
         }
+      if (want_time)
+        trans_time = sw.stop();
+      if (verbose)
+        std::cerr << "translating formula done in "
+                  << trans_time << " seconds\n";
 
       spot::twa_graph_ptr dpa = nullptr;
       switch (opt_solver)
         {
           case DET_SPLIT:
             {
+              if (want_time)
+                sw.start();
               auto tmp = to_dpa(aut);
               if (verbose)
                 std::cerr << "determinization done\nDPA has "
-                          << dpa->num_states() << " states, "
-                          << dpa->num_sets() << " colors\n";
+                          << tmp->num_states() << " states, "
+                          << tmp->num_sets() << " colors\n";
               tmp->merge_states();
+              if (want_time)
+                paritize_time = sw.stop();
               if (verbose)
                 std::cerr << "simplification done\nDPA has "
-                          << tmp->num_states() << " states\n";
+                          << tmp->num_states() << " states\n"
+                          << "determinization and simplification took "
+                          << paritize_time << " seconds\n";
+              if (want_time)
+                sw.start();
               dpa = split_2step(tmp, all_inputs);
-              if (verbose)
-                std::cerr << "split inputs and outputs done\nautomaton has "
-                          << dpa->num_states() << " states\n";
               spot::colorize_parity_here(dpa, true);
+              if (want_time)
+                split_time = sw.stop();
+              if (verbose)
+                std::cerr << "split inputs and outputs done in " << split_time
+                          << " seconds\nautomaton has "
+                          << tmp->num_states() << " states\n";
               break;
             }
           case SPLIT_DET:
             {
+              if (want_time)
+                sw.start();
               auto split = split_2step(aut, all_inputs);
+              if (want_time)
+                split_time = sw.stop();
               if (verbose)
-                std::cerr << "split inputs and outputs done\nautomaton has "
+                std::cerr << "split inputs and outputs done in " << split_time
+                          << " seconds\nautomaton has "
                           << split->num_states() << " states\n";
+              if (want_time)
+                sw.start();
               dpa = to_dpa(split);
               if (verbose)
                 std::cerr << "determinization done\nDPA has "
@@ -345,49 +440,89 @@ namespace
               dpa->merge_states();
               if (verbose)
                 std::cerr << "simplification done\nDPA has "
-                          << dpa->num_states() << " states\n";
+                          << dpa->num_states() << " states\n"
+                          << "determinization and simplification took "
+                          << paritize_time << " seconds\n";
+              if (want_time)
+                paritize_time = sw.stop();
               break;
             }
           case LAR:
+          case LAR_OLD:
             {
+              if (want_time)
+                sw.start();
               dpa = split_2step(aut, all_inputs);
               dpa->merge_states();
+              if (want_time)
+                split_time = sw.stop();
               if (verbose)
-                std::cerr << "split inputs and outputs done\nautomaton has "
+                std::cerr << "split inputs and outputs done in " << split_time
+                          << " seconds\nautomaton has "
                           << dpa->num_states() << " states\n";
-              dpa = spot::to_parity(dpa);
-              spot::reduce_parity_here(dpa, true);
+              if (want_time)
+                sw.start();
+              if (opt_solver == LAR)
+                {
+                  dpa = spot::to_parity(dpa);
+                  // reduce_parity is called by to_parity(),
+                  // but with colorization turned off.
+                  spot::colorize_parity_here(dpa, true);
+                }
+              else
+                {
+                  dpa = spot::to_parity_old(dpa);
+                  dpa = reduce_parity_here(dpa, true);
+                }
               spot::change_parity_here(dpa, spot::parity_kind_max,
                                        spot::parity_style_odd);
+              if (want_time)
+                paritize_time = sw.stop();
               if (verbose)
-                std::cerr << "LAR construction done\nDPA has "
+                std::cerr << "LAR construction done in " << paritize_time
+                          << " seconds\nDPA has "
                           << dpa->num_states() << " states, "
                           << dpa->num_sets() << " colors\n";
               break;
             }
         }
+      if (want_time)
+        sw.start();
       auto owner = complete_env(dpa);
       auto pg = spot::parity_game(dpa, owner);
+      if (want_time)
+        bgame_time = sw.stop();
       if (verbose)
-        std::cerr << "parity game built\n";
-      timer.stop();
+        std::cerr << "parity game built in " << bgame_time << " seconds\n";
 
       if (opt_print_pg)
         {
+          timer.stop();
           pg.print(std::cout);
           return 0;
         }
 
       spot::parity_game::strategy_t strategy[2];
       spot::parity_game::region_t winning_region[2];
+      if (want_time)
+        sw.start();
       pg.solve(winning_region, strategy);
+      if (want_time)
+        solve_time = sw.stop();
+      if (verbose)
+        std::cerr << "parity game solved in " << solve_time << " seconds\n";
+      timer.stop();
       if (winning_region[1].count(pg.get_init_state_number()))
         {
           std::cout << "REALIZABLE\n";
           if (!opt_real)
             {
+              if (want_time)
+                sw.start();
               auto strat_aut =
                 strat_to_aut(pg, strategy[1], dpa, all_outputs);
+              if (want_time)
+                strat2aut_time = sw.stop();
 
               // output the winning strategy
               if (opt_print_aiger)
@@ -406,6 +541,15 @@ namespace
           return 1;
         }
     }
+
+    int process_formula(spot::formula f, const char*, int) override
+    {
+      unsigned res = solve_formula(f);
+      if (opt_csv)
+        print_csv(f, res == 0);
+      return res;
+    }
+
   };
 }
 
@@ -416,6 +560,12 @@ parse_opt(int key, char* arg, struct argp_state*)
   BEGIN_EXCEPTION_PROTECT;
   switch (key)
     {
+    case OPT_ALGO:
+      opt_solver = XARGMATCH("--algo", arg, solver_args, solver_types);
+      break;
+    case OPT_CSV:
+      opt_csv = arg ? arg : "-";
+      break;
     case OPT_INPUT:
       {
         std::istringstream aps(arg);
@@ -441,15 +591,12 @@ parse_opt(int key, char* arg, struct argp_state*)
     case OPT_PRINT:
       opt_print_pg = true;
       break;
-    case OPT_ALGO:
-      opt_solver = XARGMATCH("--algo", arg, solver_args, solver_types);
+    case OPT_PRINT_AIGER:
+      opt_print_aiger = true;
       break;
     case OPT_REAL:
       opt_real = true;
       break;
-    case OPT_PRINT_AIGER:
-      opt_print_aiger = true;
-      break;
     case OPT_VERBOSE:
       verbose = true;
       break;
@@ -468,7 +615,10 @@ main(int argc, char **argv)
         exit(err);
       check_no_formula();
 
-      spot::translator trans;
+      // Setup the dictionary now, so that BuDDy's initialization is
+      // not measured in our timings.
+      spot::bdd_dict_ptr dict = spot::make_bdd_dict();
+      spot::translator trans(dict);
       ltl_processor processor(trans, input_aps, output_aps);
       return processor.run();
     });
