@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2017, 2018 Laboratoire de Recherche et Développement
+// Copyright (C) 2017-2018 Laboratoire de Recherche et Développement
 // de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -17,6 +17,7 @@
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+#include "config.h"
 #include <spot/twaalgos/cobuchi.hh>
 
 #include <spot/misc/bddlt.hh>
@@ -35,11 +36,11 @@
 #include <stack>
 #include <unordered_map>
 
-#define DEBUG 0
-#if DEBUG
-#define debug std::cerr
+#define TRACE 0
+#if TRACE
+#define trace std::cerr
 #else
-#define debug while (0) std::cout
+#define trace while (0) std::cout
 #endif
 
 namespace spot
@@ -132,7 +133,7 @@ namespace spot
         product_states* res_map_;               // States of the aug. sub. cons.
         scc_info si_;                           // SCC information.
         unsigned nb_states_;                    // Number of states.
-        unsigned was_rabin_;                    // Was it Rabin before Streett?
+        unsigned was_dnf_;                      // Was it DNF before Streett?
         std::vector<unsigned>* orig_states_;    // Match old Rabin st. from new.
         std::vector<unsigned>* orig_clauses_;   // Associated Rabin clauses.
         unsigned orig_num_st_;                  // Rabin original nb states.
@@ -147,7 +148,7 @@ namespace spot
             bv->set(state);
           unsigned clause = 0;
           unsigned state = st.first;
-          if (was_rabin_)
+          if (was_dnf_)
             {
               clause = (*orig_clauses_)[state];
               assert((int)clause >= 0);
@@ -169,7 +170,7 @@ namespace spot
               if (nca_is_inf_state[s])
                 {
                   for (auto& e : res_->out(s))
-                    e.acc = 0U;
+                    e.acc = {};
 
                   if (nca_info)
                     save_inf_nca_st(s, nca_info);
@@ -181,7 +182,7 @@ namespace spot
                       if (si_.scc_of(e.dst) == src_scc || state_based_)
                           e.acc = acc_cond::mark_t({0});
                       else
-                        e.acc = 0u;
+                        e.acc = {};
                     }
                 }
             }
@@ -193,7 +194,7 @@ namespace spot
                              const const_twa_graph_ptr ref_power,
                              std::vector<acc_cond::rs_pair>& pairs,
                              bool named_states = false,
-                             bool was_rabin = false,
+                             bool was_dnf = false,
                              unsigned orig_num_st = 0)
           : aut_(ref_prod),
             state_based_(aut_->prop_state_acc().is_true()),
@@ -205,10 +206,10 @@ namespace spot
             si_(res_, (scc_info_options::TRACK_STATES
                        | scc_info_options::TRACK_SUCCS)),
             nb_states_(res_->num_states()),
-            was_rabin_(was_rabin),
+            was_dnf_(was_dnf),
             orig_num_st_(orig_num_st ? orig_num_st : ref_prod->num_states())
         {
-          if (was_rabin)
+          if (was_dnf)
             {
               orig_states_ = ref_prod->get_named_prop<std::vector<unsigned>>
                 ("original-states");
@@ -317,7 +318,7 @@ namespace spot
       unsigned ns = si.scc_count();
       for (unsigned s = 0; s < ns; ++s)
         {
-          acc_cond::mark_t m = 0U;
+          acc_cond::mark_t m = {};
           if (si.is_rejecting_scc(s))
             m = acc_cond::mark_t{0};
           else
@@ -464,7 +465,7 @@ namespace spot
                     }
                 }
             }
-          debug << "All_states:\n" << *bv_aut_trans_ << '\n';
+          trace << "All_states:\n" << *bv_aut_trans_ << '\n';
 
           twa_graph_ptr res = make_twa_graph(aut_->get_dict());
           res->copy_ap_of(aut_);
@@ -551,14 +552,14 @@ namespace spot
                       bv_trans_mark->at(l) |= bv_aut_trans_->at(s * nc + l);
               toclean_.push_back(bv_trans_mark);
 
-              debug << "src:" << top.second;
+              trace << "src:" << top.second;
               if (named_states)
-                debug << ' ' << (*state_name)[top.second];
-              debug << '\n';
+                trace << ' ' << (*state_name)[top.second];
+              trace << '\n';
 
               for (unsigned l = 0; l < nc; ++l)
                 {
-                  debug << "l: "
+                  trace << "l: "
                         << bdd_format_formula(aut_->get_dict(), num2bdd_[l])
                         << '\n';
 
@@ -585,12 +586,12 @@ namespace spot
                                 (top.first.first + 1) % (nb_copy_ + 1)
                               : top.first.first;
 
-                  debug << "dest\n" << *bv_res << "i: " << i << '\n';
+                  trace << "dest\n" << *bv_res << "i: " << i << '\n';
                   res->new_edge(top.second,
                                 new_state(std::make_pair(i, bv_res)),
                                 num2bdd_[l]);
                 }
-              debug << '\n';
+              trace << '\n';
             }
 
           // Set rejecting states.
@@ -620,7 +621,7 @@ namespace spot
   twa_graph_ptr
   nsa_to_dca(const_twa_graph_ptr aut, bool named_states)
   {
-    debug << "NSA_to_dca\n";
+    trace << "NSA_to_dca\n";
     std::vector<acc_cond::rs_pair> pairs;
     if (!aut->acc().is_streett_like(pairs) && !aut->acc().is_parity())
       throw std::runtime_error("nsa_to_dca() only works with Streett-like or "
@@ -635,10 +636,10 @@ namespace spot
     vect_nca_info nca_info;
     nsa_to_nca(aut, named_states, &nca_info);
 
-#if DEBUG
-    debug << "PRINTING INFO\n";
+#if TRACE
+    trace << "PRINTING INFO\n";
     for (unsigned i = 0; i < nca_info.size(); ++i)
-      debug << '<' << nca_info[i]->clause_num << ',' << nca_info[i]->state_num
+      trace << '<' << nca_info[i]->clause_num << ',' << nca_info[i]->state_num
             << ',' << *nca_info[i]->all_dst << ">\n";
 #endif
 
@@ -650,7 +651,7 @@ namespace spot
   twa_graph_ptr
   dnf_to_dca(const_twa_graph_ptr aut, bool named_states)
   {
-    debug << "DNF_to_dca\n";
+    trace << "DNF_to_dca\n";
     const acc_cond::acc_code& code = aut->get_acceptance();
     if (!code.is_dnf())
       throw std::runtime_error("dnf_to_dca() only works with DNF (Rabin-like "
@@ -665,10 +666,10 @@ namespace spot
     vect_nca_info nca_info;
     dnf_to_nca(aut, false, &nca_info);
 
-#if DEBUG
-    debug << "PRINTING INFO\n";
+#if TRACE
+    trace << "PRINTING INFO\n";
     for (unsigned i = 0; i < nca_info.size(); ++i)
-      debug << '<' << nca_info[i]->clause_num << ',' << nca_info[i]->state_num
+      trace << '<' << nca_info[i]->clause_num << ',' << nca_info[i]->state_num
             << ',' << *nca_info[i]->all_dst << ">\n";
 #endif
 
