@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2012-2019 Laboratoire de Recherche et Développement
+// Copyright (C) 2012-2020 Laboratoire de Recherche et Développement
 // de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -38,8 +38,8 @@
 #include <spot/twaalgos/alternation.hh>
 #include <spot/twaalgos/parity.hh>
 #include <spot/twaalgos/cobuchi.hh>
-#include <spot/twaalgos/rabin2parity.hh>
 #include <spot/twaalgos/cleanacc.hh>
+#include <spot/twaalgos/toparity.hh>
 
 namespace spot
 {
@@ -84,6 +84,7 @@ namespace spot
         sat_states_ = opt->get("sat-states", 0);
         state_based_ = opt->get("state-based", 0);
         wdba_minimize_ = opt->get("wdba-minimize", 1);
+        gen_reduce_parity_ = opt->get("gen-reduce-parity", 1);
 
         if (sat_acc_ && sat_minimize_ == 0)
           sat_minimize_ = 1;        // Dicho.
@@ -163,6 +164,15 @@ namespace spot
                             degen_lowinit_, degen_remscc_);
   }
 
+  static void
+  force_buchi(twa_graph_ptr& a)
+  {
+    assert(a->acc().is_t());
+    acc_cond::mark_t m = a->set_buchi();
+    for (auto& e: a->edges())
+      e.acc = m;
+  }
+
   twa_graph_ptr
   postprocessor::do_scc_filter(const twa_graph_ptr& a, bool arg) const
   {
@@ -196,6 +206,8 @@ namespace spot
       tmp = SBACC_ ? do_degen(tmp) : do_degen_tba(tmp);
     if (SBACC_)
       tmp = sbacc(tmp);
+    if (type_ == BA && tmp->acc().is_t())
+      force_buchi(tmp);
     if (want_parity)
       {
         reduce_parity_here(tmp, COLORED_);
@@ -243,15 +255,25 @@ namespace spot
         bool isparity = in->acc().is_parity();
         if (isparity && in->is_existential()
             && (type_ == Generic || want_parity))
-          return reduce_parity(in);
-        if (!(want_parity && isparity))
           {
-            if (level_ == High)
-              return simplify_acceptance(in);
-            else
-              return cleanup_acceptance(in);
+            auto res = reduce_parity(in);
+            if (want_parity || gen_reduce_parity_)
+              return res;
+            // In case type_ == Generic and gen_reduce_parity_ == 0,
+            // we only return the result of reduce_parity() if it can
+            // lower the number of colors.  Otherwise,
+            // simplify_acceptance() will not do better and we return
+            // the input unchanged.  The reason for not always using
+            // the output of reduce_parity() is that is may hide
+            // symmetries between automata, as in issue #402.
+            return (res->num_sets() < in->num_sets()) ? res : in;
           }
-        return cleanup_parity(in);
+        if (want_parity && isparity)
+          return cleanup_parity(in);
+        if (level_ == High)
+          return simplify_acceptance(in);
+        else
+          return cleanup_acceptance(in);
       };
     a = simplify_acc(a);
 
@@ -263,22 +285,25 @@ namespace spot
         !(type_ == Generic && PREF_ == Any && level_ == Low))
       a = remove_alternation(a);
 
-    if ((via_gba && !a->acc().is_generalized_buchi())
-        || (want_parity && !a->acc().is_parity()))
+    if ((via_gba || (want_parity && !a->acc().is_parity()))
+        && !a->acc().is_generalized_buchi())
       {
-        twa_graph_ptr b = nullptr;
-        if (want_parity && is_deterministic(a) &&
-            !a->acc().is_generalized_buchi())
-          b = iar_maybe(a);
-        // possible only if a was deterministic and (Rabin-like or Streett-like)
-        // and we want parity and a is not a TGBA
-        // NB: on a TGBA, degeneralization is better than IAR
-        if (b)
-          a = b;
+        // If we do want a parity automaton, we can use to_parity().
+        // However (1) degeneralization is better if the input is
+        // GBA, and (2) if we want a deterministic parity automaton and the
+        // input is not deterministic, that is useless here.  We need
+        // to determinize it first, and our deterministization
+        // function only deal with TGBA as input.
+        if (want_parity && (PREF_ != Deterministic || is_deterministic(a)))
+          {
+            a = to_parity(a);
+          }
         else
-          a = to_generalized_buchi(a);
-        if (PREF_ == Any && level_ == Low)
-          a = do_scc_filter(a, true);
+          {
+            a = to_generalized_buchi(a);
+            if (PREF_ == Any && level_ == Low)
+              a = do_scc_filter(a, true);
+          }
       }
 
     if (PREF_ == Any && level_ == Low
