@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2014-2019 Laboratoire de Recherche et Développement
+// Copyright (C) 2014-2020 Laboratoire de Recherche et Développement
 // de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -158,24 +158,25 @@ namespace spot
                 // Gather all successor SCCs
                 if (track_succs)
                   for (auto& t: aut->out(*s))
-                    for (unsigned d: aut->univ_dests(t))
-                      {
-                        unsigned n = sccof_[d];
-                        if (n == num || n == -1U)
-                          continue;
-                        // If edges are cut, we are not able to
-                        // maintain proper successor information.
-                        if (filter_)
-                          switch (filter_(t, d, filter_data_))
-                            {
-                            case edge_filter_choice::keep:
-                              break;
-                            case edge_filter_choice::ignore:
-                            case edge_filter_choice::cut:
-                              continue;
-                            }
-                        succ.emplace_back(n);
-                      }
+                    if (SPOT_LIKELY(t.cond != bddfalse))
+                      for (unsigned d: aut->univ_dests(t))
+                        {
+                          unsigned n = sccof_[d];
+                          if (n == num || n == -1U)
+                            continue;
+                          // If edges are cut, we are not able to
+                          // maintain proper successor information.
+                          if (filter_)
+                            switch (filter_(t, d, filter_data_))
+                              {
+                              case edge_filter_choice::keep:
+                                break;
+                              case edge_filter_choice::ignore:
+                              case edge_filter_choice::cut:
+                                continue;
+                              }
+                          succ.emplace_back(n);
+                        }
               }
             while (*s++ != curr);
 
@@ -251,6 +252,13 @@ namespace spot
             // Fetch the values we are interested in...
             auto& e = gr.edge_storage(tr_succ);
 
+            // Skip false edges.
+            if (SPOT_UNLIKELY(e.cond == bddfalse))
+              {
+                todo_.top().out_edge = e.next_succ;
+                continue;
+              }
+
             unsigned dest = e.dst;
             if ((int) dest < 0)
               {
@@ -412,7 +420,7 @@ namespace spot
         auto& s = result[src_scc];
         for (auto& t: aut_->out(src))
           {
-            if (scc_of(t.dst) != src_scc)
+            if (scc_of(t.dst) != src_scc || SPOT_UNLIKELY(t.cond == bddfalse))
               continue;
             s.insert(t.acc);
           }
@@ -438,7 +446,7 @@ namespace spot
     return support;
   }
 
-  bool scc_info::check_scc_emptiness(unsigned n)
+  bool scc_info::check_scc_emptiness(unsigned n) const
   {
     if (SPOT_UNLIKELY(!aut_->is_existential()))
       throw std::runtime_error("scc_info::check_scc_emptiness() "
@@ -502,7 +510,7 @@ namespace spot
         bfs_queue.pop_front();
         for (auto& t: aut->out(src))
           {
-            if (filter(t))
+            if (SPOT_UNLIKELY(t.cond == bddfalse) || filter(t))
               continue;
 
             if (match(t))
@@ -558,37 +566,44 @@ namespace spot
                   };
 
     // The SCC exploration has a small optimization that can flag SCCs
-    // has accepting if they contain accepting self-loops, even if the
+    // as accepting if they contain accepting self-loops, even if the
     // SCC itself has some Fin acceptance to check.  So we have to
     // deal with this situation before we look for the more complex
     // case of satisfying the condition with a larger cycle.  We do
     // this first, because it's good to return a small cycle if we
     // can.
     const acc_cond& acccond = aut_->acc();
-    for (unsigned s: node.states())
-      for (auto& e: aut_->out(s))
-        if (e.src == e.dst && !filter(e) && acccond.accepting(e.acc))
-          {
-            // We have found an accepting self-loop.  That's the cycle
-            // part of our accepting run.
-            r->cycle.clear();
-            r->cycle.emplace_front(aut_->state_from_number(e.src),
-                                   e.cond, e.acc);
-            // Add the prefix.
-            r->prefix.clear();
-            if (e.src != init)
-              explicit_bfs_steps(aut_, init, r->prefix,
-                                 [](const twa_graph::edge_storage_t&)
-                                 {
-                                   return false; // Do not filter.
-                                 },
-                                 [&](const twa_graph::edge_storage_t& t)
-                                 {
-                                   return t.dst == e.src;
-                                 });
-            return;
-          }
-
+    unsigned num_states = aut_->num_states();
+    for (unsigned s = 0; s < num_states; ++s)
+      {
+        // We scan the entire state to find those in the SCC, because
+        // we cannot rely on TRACK_STATES being on.
+        if (scc_of(s) != scc)
+          continue;
+        for (auto& e: aut_->out(s))
+          if (e.src == e.dst && SPOT_LIKELY(e.cond != bddfalse)
+              && !filter(e) && acccond.accepting(e.acc))
+            {
+              // We have found an accepting self-loop.  That's the cycle
+              // part of our accepting run.
+              r->cycle.clear();
+              r->cycle.emplace_front(aut_->state_from_number(e.src),
+                                     e.cond, e.acc);
+              // Add the prefix.
+              r->prefix.clear();
+              if (e.src != init)
+                explicit_bfs_steps(aut_, init, r->prefix,
+                                   [](const twa_graph::edge_storage_t&)
+                                   {
+                                     return false; // Do not filter.
+                                   },
+                                   [&](const twa_graph::edge_storage_t& t)
+                                   {
+                                     return t.dst == e.src;
+                                   });
+              return;
+            }
+      }
 
     // Prefix search
 
@@ -832,4 +847,27 @@ namespace spot
 
     return res;
   }
+
+    scc_info::edge_filter_choice
+    scc_and_mark_filter::filter_scc_and_mark_
+    (const twa_graph::edge_storage_t& e, unsigned dst, void* data)
+    {
+      auto& d = *reinterpret_cast<scc_and_mark_filter*>(data);
+      if (d.lower_si_->scc_of(dst) != d.lower_scc_)
+        return scc_info::edge_filter_choice::ignore;
+      if (d.cut_sets_ & e.acc)
+        return scc_info::edge_filter_choice::cut;
+      return scc_info::edge_filter_choice::keep;
+    }
+
+    scc_info::edge_filter_choice
+    scc_and_mark_filter::filter_mark_
+    (const twa_graph::edge_storage_t& e, unsigned, void* data)
+    {
+      auto& d = *reinterpret_cast<scc_and_mark_filter*>(data);
+      if (d.cut_sets_ & e.acc)
+        return scc_info::edge_filter_choice::cut;
+      return scc_info::edge_filter_choice::keep;
+    }
+
 }
