@@ -1,6 +1,6 @@
 #!/usr/bin/python3
 # -*- mode: python; coding: utf-8 -*-
-# Copyright (C) 2017, 2018 Laboratoire de Recherche et Développement de
+# Copyright (C) 2017-2019 Laboratoire de Recherche et Développement de
 # l'EPITA.
 #
 # This file is part of Spot, a model checking library.
@@ -21,68 +21,78 @@
 import spot
 import buddy
 
-match_strings = [('is_buchi', 'is_co_buchi'),\
-                 ('is_generalized_buchi', 'is_generalized_co_buchi'),\
-                 ('is_all', 'is_none'),\
+match_strings = [('is_buchi', 'is_co_buchi'),
+                 ('is_generalized_buchi', 'is_generalized_co_buchi'),
+                 ('is_all', 'is_none'),
                  ('is_all', 'is_all'),
                  ('is_buchi', 'is_all')]
 
 # existential and universal are dual
 # deterministic is self-dual
+
+
 def dualtype(aut, dual):
-  if dual.acc().is_none(): return True
-  return (not spot.is_deterministic(aut) or spot.is_deterministic(dual))\
-         and (spot.is_universal(dual) or not aut.is_existential())\
-         and (dual.is_existential() or not spot.is_universal(aut))
+    if dual.acc().is_none():
+        return True
+    return (not spot.is_deterministic(aut) or spot.is_deterministic(dual))\
+        and (spot.is_universal(dual) or not aut.is_existential())\
+        and (dual.is_existential() or not spot.is_universal(aut))
+
 
 def produce_phi(rg, n):
-  phi = []
-  while len(phi) < n:
-    phi.append(rg.next())
-  return phi
+    phi = []
+    while len(phi) < n:
+        phi.append(rg.next())
+    return phi
+
 
 def produce_automaton(phi):
-  aut = []
-  for f in phi:
-    aut.append(spot.translate(f))
-  return aut
+    aut = []
+    for f in phi:
+        aut.append(spot.translate(f))
+    return aut
 
-def test_aut(aut, d = None):
-  if d is None:
-    d = spot.dualize(aut)
-  aa = aut.acc()
-  da = d.acc()
 
-  complete = spot.is_complete(aut)
-  univ = aut.is_univ_dest(aut.get_init_state_number())
-  an = aut.num_states()
-  dn = d.num_states()
+def test_aut(aut, d=None):
+    if d is None:
+        d = spot.dualize(aut)
+    aa = aut.acc()
+    da = d.acc()
 
-  if not dualtype(aut, d):
-    return (False, 'Incorrect transition mode resulting of dual')
-  for p in match_strings:
-    if ((getattr(aa, p[0])() and getattr(da, p[1])())\
-      or (getattr(aa, p[1])() and getattr(da, p[0])())):
-      return (True, '')
-  return (False, 'Incorrect acceptance type dual')
+    complete = spot.is_complete(aut)
+    univ = aut.is_univ_dest(aut.get_init_state_number())
+    an = aut.num_states()
+    dn = d.num_states()
+
+    if not dualtype(aut, d):
+        return (False, 'Incorrect transition mode resulting of dual')
+    for p in match_strings:
+        if ((getattr(aa, p[0])() and getattr(da, p[1])())
+                or (getattr(aa, p[1])() and getattr(da, p[0])())):
+            return (True, '')
+    return (False, 'Incorrect acceptance type dual')
 
 # Tests that a (deterministic) automaton and its complement have complementary
 # languages.
 # FIXME This test could be extended to non-deterministic automata with a
 # dealternization procedure.
+
+
 def test_complement(aut):
-  assert aut.is_deterministic()
-  d = spot.dualize(aut)
-  s = spot.product_or(aut, d)
-  assert spot.dualize(s).is_empty()
+    assert aut.is_deterministic()
+    d = spot.dualize(aut)
+    s = spot.product_or(aut, d)
+    assert spot.dualize(s).is_empty()
+
 
 def test_assert(a, d=None):
-  t = test_aut(a, d)
-  if not t[0]:
-    print (t[1])
-    print (a.to_str('hoa'))
-    print (spot.dualize(a).to_str('hoa'))
-    assert False
+    t = test_aut(a, d)
+    if not t[0]:
+        print (t[1])
+        print (a.to_str('hoa'))
+        print (spot.dualize(a).to_str('hoa'))
+        assert False
+
 
 aut = spot.translate('a')
 
@@ -553,7 +563,7 @@ dual = spot.dualize(aut)
 h = dual.to_str('hoa')
 
 assert h == """HOA: v1
-States: 6
+States: 5
 Start: 0
 AP: 2 "a" "b"
 acc-name: co-Buchi
@@ -561,27 +571,24 @@ Acceptance: 1 Fin(0)
 properties: trans-labels explicit-labels state-acc complete
 properties: deterministic univ-branch
 --BODY--
-State: 0
-[0] 1
-[!0] 1&2
-State: 1 {0}
-[0&1] 1
-[0&!1] 4
-[!0&!1] 2&4
-[!0&1] 1&2
+State: 0 {0}
+[0&1] 0
+[0&!1] 1
+[!0&!1] 1&2
+[!0&1] 0&2
+State: 1
+[0&1] 0
+[0&!1] 1
+[!0&!1] 1&2
+[!0&1] 0&2
 State: 2
 [!0&1] 3
-[0 | !1] 5
+[0 | !1] 4
 State: 3 {0}
 [!0] 3
-[0] 5
+[0] 4
 State: 4
-[0&1] 1
-[0&!1] 4
-[!0&!1] 2&4
-[!0&1] 1&2
-State: 5
-[t] 5
+[t] 4
 --END--"""
 
 opts = spot.option_map()
@@ -594,8 +601,8 @@ spot.srand(0)
 rg = spot.randltlgenerator(2, opts)
 
 for a in produce_automaton(produce_phi(rg, 1000)):
-  test_assert(a)
-  test_assert(spot.dualize(a), spot.dualize(spot.dualize(a)))
+    test_assert(a)
+    test_assert(spot.dualize(a), spot.dualize(spot.dualize(a)))
 
 aut = spot.automaton("""
 HOA: v1
@@ -609,4 +616,4 @@ State: 0
 test_complement(aut)
 
 for a in spot.automata('randaut -A \"random 0..6\" -H -D -n50 4|'):
-  test_complement(a)
+    test_complement(a)
